/*
Example code for Benewake TFMini time-of-flight distance sensor. 
by Peter Jansen (December 11/2017)
This example code is in the public domain.

This example communicates to the TFMini using a SoftwareSerial port at 115200, 
while communicating the distance results through the default Arduino hardware
Serial debug port. 

SoftwareSerial for some boards can be unreliable at high speeds (such as 115200). 
The driver includes some limited error detection and automatic retries, that
means it can generally work with SoftwareSerial on (for example) an UNO without
the end-user noticing many communications glitches, as long as a constant refresh
rate is not required. 

The (UNO) circuit:
 * Uno RX is digital pin 10 (connect to TX of TF Mini)
 * Uno TX is digital pin 11 (connect to RX of TF Mini)

THIS SOFTWARE IS PROVIDED ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  
*/

#include <SoftwareSerial.h>
#include <Wire.h>
#include "TFMini.h"

#define I2C_SLAVE_ADDR  0x70  //Standard MaxsonarI2CXL address 0x70 

uint16_t distance = 0;      // Distance value in 16 bit word
uint16_t strength = 0;
uint16_t DistSum = 0;
uint16_t DistAverage = 0;
uint16_t StrengthSum = 0;
uint16_t StrengthAverage = 0;
byte SampleCount;

byte received;
uint8_t LED = 13;

// Setup TFmini software serial port 
SoftwareSerial TFminiSerial(10, 11);      // Uno RX (TFMINI TX), Uno TX (TFMINI RX)
TFMini tfmini;

void setup() {
  // Step 1: Initialize hardware serial port (serial debug port)
  Serial.begin(115200);     
  Serial.println ("Initializing...");

  // Step 2: Initialize the data rate for the TFmini SoftwareSerial port
  // TFminiSerial.begin(TFMINI_BAUDRATE);
  TFminiSerial.begin(57600);    // max. 57600 for 3.3V 8MHz pro Mini
  
  // Step 3: Initialize the TF Mini sensor
  tfmini.begin(&TFminiSerial); 

  Wire.begin(I2C_SLAVE_ADDR);
  Wire.onReceive(receiveEvent);   // register event
  Wire.onRequest(requestEvent);

  pinMode (LED, OUTPUT);   
}

/*
 The default I2C-Address of the sensor is 0x70. 
 To perform a range measurement you must send the "Take Range Reading” command byte 0x51
 */
void receiveEvent(int howMany) {
  while (Wire.available()) {
    {         
    received = Wire.read(); 
    if (received == 0x51)
      {       
      DistAverage = DistSum / SampleCount;
      StrengthAverage = StrengthSum / SampleCount;
/*
      if ( StrengthAverage < 30 ) {
        DistAverage = 12000;
      }
*/
      SampleCount = 0;
      DistSum = 0;
      StrengthSum = 0;
      }
    }
  }
}

void requestEvent() 
  {
  // Serial.write("D ");
  // Serial.println (DistAverage);
  
  Wire.write ( highByte(DistAverage) );
  Wire.write ( lowByte (DistAverage) );
  
  digitalWrite(LED, !digitalRead(LED));    
  }

void loop() {
  
        /* Set Baudrate
        uint8_t Cmd1[24] = {0x42, 0x57, 0x02, 0x00, 0x00, 0x00, 0x01, 0x02,
                            0x42, 0x57, 0x02, 0x00, 0x00, 0x00, 0x05, 0x08,      // 0x02: 19200   0x03: 38400 0x05: 57600  0x06: 115200
                            0x42, 0x57, 0x02, 0x01, 0x00, 0x00, 0x00, 0x02};

        for (int i= 0; i<24; i++)
            {
            TFminiSerial.write ( Cmd1[i] ); 
            }
        TFminiSerial.flush();
        while (1);
        */
        
  // Take one TF Mini distance measurement
  distance = tfmini.getDistance();
  strength = tfmini.getRecentSignalStrength();

  // Serial.print (distance);
  // Serial.print (" ");
  // Serial.println (strength);
  
  DistSum += distance;
  StrengthSum += strength;
  SampleCount++;
}

